Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity Beam_Cycle_Reset is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Beam_Gate		: in std_logic;
		 
		 Beam_Cyc_Reset	: out std_logic
		);
end Beam_Cycle_Reset;
	
architecture main of Beam_Cycle_Reset is

signal beam_start	: std_logic;

signal timer		: std_logic_vector(23 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, idle, cycle_active, send_reset);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

sync_Beam_Gate: led
port map(sig_in => Beam_Gate, clk => clk, clrn => nreset, pls_out => beam_start);
	

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= init;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, beam_start, timer)
begin
	-- default outputs --
	timer_ena 		<= '0'; 
	Beam_Cyc_Reset	<= '0';
		
	case CS is
		when init   =>				NS <= idle;
			Beam_Cyc_Reset	<= '1';
		
		when idle   =>
			if (beam_start = '1') 	then	NS <= cycle_active;
			else							NS <= idle;
			end if;
	
		when cycle_active  =>
			timer_ena	<= '1';
			if   (timer = X"30D400") then	NS <= send_reset; -- time out at 40 ms
			else							NS <= cycle_active;
			end if;
					
		when send_reset  =>					NS <= idle;
			Beam_Cyc_Reset	<= '1';
			
		when others =>						NS <= idle;
	end case; 
			
end process;

end main;