Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity Combination_Beam_Gate is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Beam_Gate		: in std_logic;
		 No_Beam_1		: in std_logic;
		 
		 Beam_Gate_2	: out std_logic
		);
end Combination_Beam_Gate;
	
architecture main of Combination_Beam_Gate is

signal beam_start	: std_logic;
signal syncd_Gate   : std_logic;
signal syncd_No_Beam   : std_logic;


signal timer		: std_logic_vector(23 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, idle, cycle_start, cycle_active, send_reset);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

sync_Beam_Gate: led
port map(sig_in => Beam_Gate, clk => clk, clrn => nreset, pls_out => beam_start);

process(clk, nreset, Beam_Gate, No_Beam_1)
begin
	if (nreset = '0') then syncd_Gate <= '0'; syncd_No_Beam <= '0';
	elsif rising_edge(clk) then
		syncd_Gate    <= Beam_Gate;
		syncd_No_Beam <= No_Beam_1;
	end if;
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= init;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, beam_start, timer, syncd_Gate, syncd_No_Beam)
begin
	-- default outputs --
	timer_ena 		<= '0'; 
	Beam_Gate_2		<= '0';
		
	case CS is
		when init   =>				NS <= idle;
		
		when idle   =>
			if (beam_start = '1') 	then	NS <= cycle_start;
			else							NS <= idle;
			end if;
	
		when cycle_start  =>
			Beam_Gate_2		<= '1';
			timer_ena		<= '1';
			if   (timer = X"027100") then	NS <= cycle_active; -- time out at 2 ms
			else							NS <= cycle_start;
			end if;

		when cycle_active  =>
			Beam_Gate_2		<= '1';
			timer_ena		<= '1';
			
			if   (syncd_No_Beam = '1') then	NS <= send_reset;
			elsif(syncd_Gate = '0')    then NS <= send_reset;
			elsif(timer = X"30D400")   then	NS <= send_reset; -- time out at 40 ms
			else							NS <= cycle_active;
			end if;
			
		when send_reset  =>					NS <= idle;
			
		when others =>						NS <= idle;
	end case; 
			
end process;

end main;