--------------------------------------------------------------------
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my definitions and entities

-------------------------------------------------------------------- 

entity DiagnosticsMux is
	port(
			clk   		: in std_logic;
			nreset		: in std_logic;
			DiagSel		: in std_logic_vector(3 downto 0);
			
			Mem_Addr	: in std_logic_vector(17 downto 0);
			Mem_Data	: in std_logic_vector(15 downto 0);
			Mem_WEn		: in std_logic;
			Mem_OEn		: in std_logic;
			Mem_CEn		: in std_logic;
			
			
			ADC1		: in std_logic_vector(13 downto 0);
			ADC1_Clk	: in std_logic;
			ADC2		: in std_logic_vector(13 downto 0);
			ADC2_Clk	: in std_logic;
			DAC0		: in std_logic_vector(13 downto 0);
			DAC0_Clk	: in std_logic;
			MDAC1		: in std_logic_vector(11 downto 0);
			MDAC1_Strb	: in std_logic;
			MDAC2		: in std_logic_vector(11 downto 0);
			MDAC2_Strb	: in std_logic;
			Aux_Monitor : in std_logic_vector(15 downto 0);
			
			----------
			Addr_Out	: out std_logic_vector(17 downto 0);
			Data_Out	: out std_logic_vector(15 downto 0);
			Drive_Bus	: out std_logic;
			SRAM_WEn	: out std_logic;
			SRAM_OEn	: out std_logic;
			SRAM_CEn	: out std_logic;
			
			Logic_Analy_Clk	: out std_logic
			
		);
end DiagnosticsMux;
	
architecture main of DiagnosticsMux is
	
	signal timer			: std_logic_vector(3 downto 0);
	signal ena_timer		: std_logic;
	signal default_clk		: std_logic;	

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(rst_timer, run_timer, strobe_clk);
	signal CS, NS  : STATE_TYPE;
	
--===================================================================
-- 			B	E	G	I	N	
--===================================================================
begin 
		
DIAG_MUX_PROCESS:
process(clk, nreset, DiagSel, Mem_CEn, Mem_Addr, Mem_Data, Mem_OEn,
		Mem_WEn, ADC1, ADC1_Clk, ADC2, ADC2_Clk, DAC0, DAC0_Clk,
		MDAC1, MDAC1_Strb, MDAC2, MDAC2_Strb, Aux_Monitor, default_clk)
begin
	if   (Mem_CEn = '0') then
		Addr_Out	<= Mem_Addr;
		Data_Out	<= Mem_Data;
		Drive_Bus	<= Mem_OEn;
		SRAM_WEn	<= Mem_WEn;
		SRAM_OEn	<= Mem_OEn;
		SRAM_CEn	<= Mem_CEn;		
		Logic_Analy_Clk	<= '0';
		
	else
		Addr_Out	<= Mem_Addr;
		Drive_Bus	<= '1';
		SRAM_WEn	<= '1';
		SRAM_OEn	<= '1'; -- SRAM may not drive bus now.
		SRAM_CEn	<= Mem_CEn;		
		
		case DiagSel is
			when X"0" =>
				Data_Out		<= "00" & ADC1;
				Logic_Analy_Clk	<= ADC1_Clk;
			when X"1" =>
                Data_Out		<= "00" & ADC2;		
                Logic_Analy_Clk	<= ADC2_Clk;	
			when X"2" =>
                Data_Out		<= "00" & DAC0;		
                Logic_Analy_Clk	<= DAC0_Clk;
			when X"3" =>
                Data_Out		<= "0000" & MDAC1;		
                Logic_Analy_Clk	<= MDAC1_Strb;
			when X"4" =>
                Data_Out		<= "0000" & MDAC2;		
                Logic_Analy_Clk	<= MDAC2_Strb;            
			when X"5" =>
                Data_Out		<= Aux_Monitor;		
                Logic_Analy_Clk	<= default_clk;
 
 			when others =>
				Data_Out		<= X"BAD1";
				Logic_Analy_Clk	<= default_clk;
		end case;
	end if;

end process;	

------------
TIMER_PROCESS:
process(clk, nreset, ena_timer)
begin
	if   (nreset = '0') then timer <= (others => '0');
	elsif rising_edge(clk) then
		if   (ena_timer = '1') then timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

------------
TIMER_STATE_MACHINE:
process (clk, nreset)	
begin
	if(nreset='0')              then CS <= rst_timer;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; 

sm_combinatorial: process (CS, timer)
begin
	ena_timer 	<= '0';
	default_clk <= '0';
	
	case CS is
		when rst_timer   =>				NS <= run_timer;
			ena_timer 	<= '0';		 
		-----
		when run_timer =>			
			ena_timer 	<= '1';
			
			if   (timer = X"C") then  	NS <= strobe_clk;
			else						NS <= run_timer;
			end if;			
		-----
		when strobe_clk =>          	NS <= run_timer;
			ena_timer 	<= '0';
			default_clk <= '1';		
		-----	
		when others =>					NS <= rst_timer;
	end case;
end process;

end main;

