
Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity MIPL_Method_2v2 is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Ref_minus_Pherr	: in std_logic_vector(13 downto 0); 
		 Update_Strobe		: in std_logic;
		 Run_MIPL 			: in std_logic;
		 End_MIPL			: in std_logic;
		 MIPL_Gain			: in std_logic_vector(13 downto 0);
		 Integ_Gain  		: in std_logic_vector(13 downto 0);
		 NO_BEAM_OFFSET 	: in std_logic_vector(13 downto 0);
		 
		 MDAC_Gain_2		: out std_logic_vector(15 downto 0);
		 FeedBack_Out  		: out std_logic_vector(13 downto 0);
		 DAC_CLK_Out		: out std_logic
		);
end MIPL_Method_2v2;
	
architecture main of MIPL_Method_2v2 is

-- Signed Value of inputs ----------------------------------
	signal ref_minus_pherr_2s	: std_logic_vector(13 downto 0);
	signal MIPL_Gain_2s			: std_logic_vector(13 downto 0);
	signal Integ_Gain_2s		: std_logic_vector(13 downto 0);
	signal BASE_2s				: std_logic_vector(13 downto 0);
	
	signal product0_2s	: std_logic_vector(27 downto 0);
	signal product_2s	: std_logic_vector(27 downto 0);
	signal sum1_2s		: std_logic_vector(13 downto 0);
	signal sum2_2s		: std_logic_vector(13 downto 0);
	signal sum3_2s		: std_logic_vector(13 downto 0);
	signal acc_reg		: std_logic_vector(13 downto 0);

-- Output Register -----------------------------------------
	signal	dac_fb_out	: std_logic_vector(13 downto 0);
	signal  clr_acc		: std_logic;
	signal	get_fb		: std_logic;
	signal	setup_dac	: std_logic;
	signal  strb_dac	: std_logic;
	
	signal  run			: std_logic;
	signal  prev_out	: std_logic_vector(13 downto 0);
	signal  clr_prev	: std_logic;
	signal  get_prev	: std_logic;
	signal  avg_sum		: std_logic_vector(13 downto 0);
----------------------------------------------------------------
	component mult14signed
		PORT
		(
			dataa		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
			datab		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
			result		: OUT STD_LOGIC_VECTOR (27 DOWNTO 0)
		);
	end component;
	
	component add14signed
	PORT
	(
		dataa		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (13 DOWNTO 0)
	);
	end component;
----------------------------------------------------------------

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, wait_1, get_feedback, latch_dac, strobe_dac);
	signal CS, NS  : STATE_TYPE;
	

-- ## MAIN #########################################################
begin 

run <= Run_MIPL;

-- Convert Offset Binary Values to 2's Complement ------------------
-- Gate off Phase to Reference Error Until MIPL --------------------

ENABLE_MIPL_FEEDBACK:
process (clk, nreset, run, Ref_minus_Pherr)
begin
	if   (nreset = '0') then ref_minus_pherr_2s <= (others => '0');
	elsif rising_edge(clk) then
		if   (run = '0') then 	ref_minus_pherr_2s 	<= (others => '0');
		else ref_minus_pherr_2s <= (not Ref_minus_Pherr(13)) & Ref_minus_Pherr(12 downto 0);
		end if;
	end if;
end process;
		
	MIPL_Gain_2s 	<= not MIPL_Gain(13)      & MIPL_Gain(12 downto 0);	
	Integ_Gain_2s 	<= not Integ_Gain(13)     & Integ_Gain(12 downto 0);
	BASE_2s		  	<= not NO_BEAM_OFFSET(13) & NO_BEAM_OFFSET(12 downto 0);	
 
	G1xFEEDBACK : mult14signed 
	port map (
				dataa	 => ref_minus_pherr_2s,
				datab	 => MIPL_Gain_2s,
				result	 => product0_2s(27 downto 0)
			 );
			   
	G2xFEEDBACK : mult14signed 
	port map (
				dataa	 => ref_minus_pherr_2s,
				datab	 => Integ_Gain_2s,
				result	 => product_2s(27 downto 0)
			 );
			 
	ACCUMULATOR : add14signed 
	port map (
				dataa	 => product_2s(27 downto 14), -- !! Divide by 16,384
				datab	 => acc_reg,
				result	 => sum1_2s(13 downto 0)
			 );
			
	ADD_TERMS : add14signed 
	port map (
				dataa	 => acc_reg,
				datab	 => product0_2s(27) & product0_2s(21 downto 9), -- !! Divide by 512
				result	 => sum2_2s(13 downto 0)
			 );

	AVERAGE_OUT : add14signed 
	port map (
				dataa	 => sum2_2s,
				datab	 => prev_out,
				result	 => avg_sum(13 downto 0)
			 );

	ADD_to_BASE : add14signed 
	port map (
				dataa	 => avg_sum(13) & avg_sum(13 downto 1),
				datab	 => BASE_2s,
				result	 => sum3_2s(13 downto 0)
			 );
			 
			 
	ACCUMULATOR_REGISTER:
	process(clk, nreset, get_fb, sum1_2s)
	begin
		if  (nreset = '0') then acc_reg <= (others => '0');
		elsif rising_edge(clk) then
			if   (get_fb = '1')  then acc_reg <= sum1_2s;
			elsif(clr_acc = '1') then acc_reg <= (others => '0');
			end if;
		end if;
	end process;

	AVERAGE_PREVIOUS_VALUE_REGISTER:
	process(clk, nreset, clr_prev, get_prev)
	begin
		if   (nreset = '0') then prev_out <= (others => '0');
		elsif rising_edge(clk) then
			if   (clr_prev = '1') then prev_out <= (others => '0');
			elsif(get_prev = '1') then prev_out <= sum2_2s;
			end if;
		end if;	
	end process;
	
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset, End_MIPL)	
	begin
		if((nreset='0') OR (End_MIPL = '1')) then CS <= idle;
		elsif rising_edge(clk)		then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: 
	process (CS, run, Update_Strobe)
	begin
		-- defaults -------
		get_fb 		<= '0'; 
		setup_dac	<= '0'; 
		clr_acc		<= '0';
		strb_dac	<= '0';	
		clr_prev	<= '0';	
		get_prev	<= '0';
		
		case CS is
			when idle =>						NS <= wait_1;
				clr_prev	<= '1';	
				clr_acc		<= '1';
--				if   (run = '1') 	   then 	NS <= wait_1;
--				else							NS <= idle;
--				end if;

			when wait_1 => 
				if(Update_Strobe = '1') then	NS <= get_feedback;
				else							NS <= wait_1;
				end if;
			
			when get_feedback => 				NS <= latch_dac;
				get_fb 		<= '1';	-- latch sum1_2s into the accumulator
						
			when latch_dac =>					NS <= strobe_dac;
				setup_dac 	<= '1'; -- latch new value into FeedBack_Out

			when strobe_dac =>					NS <= wait_1;
				get_prev	<= '1';
				strb_dac	<= '1'; -- strobe Feedback into DAC
			
			when others => 						NS <= idle;
		end case; 
				
	end process; -- sm_combinatorial

	
-- Syncronized Outputs ----------------------------

	process(clk, nreset, sum2_2s)
	begin
		if   (nreset = '0') then dac_fb_out <= "10000000000000";
		elsif rising_edge(clk) then
			if(setup_dac = '1') then
				-- convert back to 14 bit offset binary --
				dac_fb_out <= not sum3_2s(13) & sum3_2s(12 downto 0);
			end if;
		end if;
	end process;

-- ==== NEW NEW NEW NEW NEW  ========  Added Jan 23 2012
	process(clk, nreset, strb_dac)
	begin
		if   (nreset = '0')    then DAC_CLK_Out <= '0';
		elsif rising_edge(clk) then DAC_CLK_Out <= strb_dac; -- MOVED DAC_CLK_Out 1 CLOCK LATER AFTER BUFFER
		end if;
	end process;

	MDAC_Gain_2 	<= X"0800"; -- held at zero for Method 2
	FeedBack_Out 	<= dac_fb_out;
	
end main;

