Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity No_Beam_Logic is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 No_Beam_In		: in std_logic;
		 
		 No_Beam_Out	: out std_logic;
		 HOLD_Acc_PD	: out std_logic		 
		);
end No_Beam_Logic;
	
architecture main of No_Beam_Logic is

signal no_beam		: std_logic;
signal q0			: std_logic;

signal timer		: std_logic_vector(19 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, beam, hold, delay);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

sync_NO_BEAM: 
process(clk, nreset, No_Beam_In, q0)
begin
	if   (nreset = '0')   then 
		no_beam <= '0';
		q0		<= '0';
	elsif rising_edge(clk) then 
		q0		<= No_Beam_In;
		no_beam <= q0;
	end if;
end process;
	
	
PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, no_beam, timer)
begin
	-- default outputs --
	timer_ena 	<= '0'; 
	HOLD_Acc_PD <= '0';
	No_Beam_Out <= '0';
		
	case CS is
		when idle   => -- no beam
			No_Beam_Out <= '1';
			if (no_beam = '0') 	then NS <= beam;
			else					 NS <= idle;
			end if;
		
		when beam =>
			if (no_beam = '1') 	then NS <= hold;
			else					 NS <= beam;
			end if;
			
		when hold=>					 NS <= delay; --<-- bypassing  if statement
			HOLD_Acc_PD <= '1';  -- holds Accel PD gain before No_Beam sets gain to zero
			timer_ena	<= '1';
			
			if   (no_beam = '0') then		NS <= beam; -- extends delay between holding and no_beam signal
			elsif(timer = X"00008") then	NS <= delay;
			else							NS <= hold;
			end if;
		
		when delay =>
			No_Beam_Out <= '1'; -- sets gain to zero
			HOLD_Acc_PD <= '1'; -- but Accel PD error held just before summing amp
			timer_ena	<= '1';
			if   (no_beam = '0') then		NS <= beam;
			elsif(timer = X"13880") then	NS <= idle; -- hold Accel PD error for 1ms
			else							NS <= delay;
			end if;

		when others =>				NS <= idle;
	end case; 
			
end process;

end main;