--------------------------------------------------------------------
--  This routine will generate a curve from ROM memory.  The ROM
--  memory is written during compile time from the following file:
--
--         "curve_mem1.mif" <-- 256, 11 bit words
--


Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity PhaseErrorReference2 is
	port(clk        		: in std_logic;
		 nreset     		: in std_logic;
		 Run_MIPL 			: in std_logic;
		 intv_clks  		: in std_logic_vector(15 downto 0);
		 Phase_Err_In  		: in std_logic_vector(13 downto 0);
		 Phase_Err_Strb 	: in std_logic;
		 Curve_Start_Offset	: in std_logic_vector(13 downto 0);
		 Curve_Start_Scale	: in std_logic_vector(13 downto 0);
		 Beam_Gate			: in std_logic;
		 Gate_MIPL_Always	: in std_logic;
		 
		 Ref_minus_Pherr	: out std_logic_vector(13 downto 0);
		 Phase_Err_Out		: out std_logic_vector(27 downto 0);
		 Update_Strobe		: out std_logic
		);
end PhaseErrorReference2;
	
architecture main of PhaseErrorReference2 is

------------------------------------------------------------
-- ALTERA COMPONENTS ---------------------------------------
-- The following components are the ROM memory and a multiplier
-- They were created using the LPM Megafunction Wizard.
-- The following VHDL files were generated and must be included 
-- in the project.
--                              curve_mem4.vhd
--                            mult14signed.vhd
--
-- The Megafunction Wizard also generate *.cmp files for each
-- from which I cut and pasted the component declarations below.
----------------------------------------------------------------
	component curve_mem4
		PORT
		(
			address		: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
			clock		: IN STD_LOGIC ;
			data		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
			wren		: IN STD_LOGIC ;
			q			: OUT STD_LOGIC_VECTOR (13 DOWNTO 0)
		);
	end component;

-- ROM Curve intermediate signals -------------------------------
	signal curve   		: std_logic_vector(13 downto 0);
	
-----------------------------------------------------------------
	component mult14signed
		PORT
		(
			dataa		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
			datab		: IN STD_LOGIC_VECTOR (13 DOWNTO 0);
			result		: OUT STD_LOGIC_VECTOR (27 DOWNTO 0)
		);
	end component;
	
-- Multiplier intermediate signal ------------------------------
	signal product_2s   		: std_logic_vector(27 downto 0);
	signal scaled_phase_err_2s 	: std_logic_vector(27 downto 0);

-----------------------------------------------------------------
	component sub28signed
		PORT
		(
			dataa		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);
			datab		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);
			result		: OUT STD_LOGIC_VECTOR (27 DOWNTO 0)
		);
	end component;
	
-----------------------------------------------------------------
-- Register to capture the initial offset value ------------
	signal	pherr_latch_2s	: std_logic_vector(13 downto 0);

-- Pipeline Delay Registers ------------
	signal  pherr_reg1_2s : std_logic_vector(13 downto 0);
	signal  phase_err_2s 	: std_logic_vector(27 downto 0);

	signal  phase_err_in_2s	: std_logic_vector(13 downto 0);
	
	
	signal strb_delay	: std_logic_vector(1 downto 0);
		
-- Multiplier Result Register --------------------------------
	signal  ref_trajectory_2s	: std_logic_vector(27 downto 0);
	
-- Reference minus the Phase Error --------------------------------
	signal  ref_minus_pherr_2s	: std_logic_vector(27 downto 0);
		
----------------------------------------------------------------
-----------------------------------------------------------------

-- ADDR_CNT Counter/Timer declarations ---------------------------
-- The output of this counter is the address pointer
	signal nclr_adr : std_logic;  -- output of sm
	signal ld_adr	: std_logic;
	signal inc_adr  : std_logic;  -- output of sm
	signal end_flag : std_logic;  -- output of ADDR_CNT, input to sm
	signal last_adr : std_logic_vector(7 downto 0);  -- input to ADDR_CNT
	signal addr     : std_logic_vector(7 downto 0);  -- output of ADDR_CNT
	
	-- INTERVAL_CNT declarations ------------------------------
	signal nclr_cnt : std_logic; -- output of sm
	signal smp_flag : std_logic; -- output of INTERVAL_CNT, input to sm

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is (idle, wait_1, nextadr, hold);
	signal CS, NS  : STATE_TYPE;
	
	
	
-- ## MAIN #########################################################
begin 

------------------------------------------------------------
	phase_err_in_2s <= (NOT Phase_Err_In(13)) & Phase_Err_In(12 downto 0);

--------------------------------------------------------------------------------
-- Phase Error Pipeline ------------------------------------

	PHASE_ERROR_PIPELINE_REG_IN:
	process(clk, nreset, Phase_Err_Strb, phase_err_in_2s)
	begin
		if   (nreset = '0') 		  then  pherr_reg1_2s <= (others => '0');
		elsif rising_edge(clk) 	then
			if (Phase_Err_Strb = '1') then 	pherr_reg1_2s <= phase_err_in_2s;
			end if;
		end if;
	end process;
	
	--Scale Phase Error for Adjusted Curve Start Approach --	
	PHASE_ERR_x_CURVE_SCALE : mult14signed 
	port map (
				dataa	 => pherr_reg1_2s,
				datab	 => (NOT Curve_Start_Scale(13)) & Curve_Start_Scale(12 downto 0),
				result	 => scaled_phase_err_2s(27 downto 0)
			 );			 			 
	
	-- Divide by 8192 and Phase Error Result Register --
	PHASE_ERROR_PIPELINE_REG_OUT:
	process(clk, nreset, scaled_phase_err_2s)
	begin
		if   (nreset = '0') 	then  phase_err_2s <= (others => '0');
		elsif rising_edge(clk) 	then  phase_err_2s <= scaled_phase_err_2s;
		end if;
	end process;
	
	Phase_Err_Out <= (NOT phase_err_2s(27)) & phase_err_2s(26 downto 0); -- 28 bit Offset Binary Value	

------------------------------------------------------------
-- Phase Error Latch ---------------------------------------
	PHASE_ERROR_LATCH: -- Latch the phase error when Run_MIPL -> '1'
	process(clk, nreset, Phase_Err_Strb, Run_MIPL, phase_err_in_2s)
	begin
		if   (nreset = '0') 	then pherr_latch_2s <= "00000000000000";

		elsif rising_edge(clk) 	then
			if ((Phase_Err_Strb = '1') AND (Run_MIPL = '0')) then 	
				pherr_latch_2s <= phase_err_in_2s;
			end if;
		end if;
	end process;
	
-- Curve Memory --------------------------------------------
	CURVE_MEM: curve_mem4
	port map(address => addr, 
			 clock => clk, 
			 data => (others => '0'),
			 wren => '0',
			 q => curve(13 downto 0)
			);

	-- Scale Initial Phase Error to Curve --		
	SCALE_PHASE_ERR : mult14signed 
	port map (
				dataa	 => pherr_latch_2s(13 downto 0),
				datab	 => (NOT curve(13)) & curve(12 downto 0),
				result	 => product_2s(27 downto 0)
			 );
			 			 
	-- Divide by 8192 and Reference Result Register --
	MULT_RESULT_REG:
	process(clk, nreset, product_2s)
	begin
		if   (nreset = '0') 	then  ref_trajectory_2s <= (others => '0');
		elsif rising_edge(clk) 	then  ref_trajectory_2s <= product_2s;
		end if;
	end process;
			
-------------------------------------------------------------------
-- Subtract phase error trajectory from phase error ---------------
	ERROR_PHERR_TO_REF: sub28signed
	port map (
				dataa	 => ref_trajectory_2s,
				datab	 => phase_err_2s, 
				result	 => ref_minus_pherr_2s(27 downto 0)
			 );
			 
--------------------------------------------------------------
process(Beam_Gate, Gate_MIPL_Always, ref_minus_pherr_2s)
begin
if((Beam_Gate = '1') OR (Gate_MIPL_Always = '1')) then
	Ref_minus_Pherr <= (NOT ref_minus_pherr_2s(27)) & ref_minus_pherr_2s(25 downto 13);
		--^ 14 Bit Offset Binary Value and Divide by 8192 --
else
	Ref_minus_Pherr <= (others => '0');
end if;

end process;
--------------------------------------------------------------

--------------------------------------------------------------
--Memory Address Pointer -------------------------------------		
	ADDR_CNT: counter  -- address sequencer
	-- The output of this counter is the address pointer
	generic map(WID8)
	port map ( clk=>clk, clrn=>nclr_adr, ena=>inc_adr, load=>ld_adr, updn=>'1',
			   ld_data  => Curve_Start_Offset(7 downto 0),
			   flag_cnt => last_adr,
			   flag_out => end_flag,
			   cnt_out  => addr
			 );
	last_adr <= X"FE"; -- X"FD" changed Aug 10
	
	INTERVAL_CNT: counter  -- Defines the curve sample interval
	generic map(WID16)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => intv_clks,
			   flag_out => smp_flag,
			   cnt_out  => open
			 );

---------------------------------------------------------------		
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, smp_flag, end_flag, Run_MIPL)
	begin
		nclr_cnt <= '0'; -- counter is reset 
		nclr_adr <= '1'; -- Changed Aug 10, loading instead of clearing -- counter is reset 
		inc_adr  <= '0'; -- not incrementing the address pointer
		ld_adr	 <= '0';
		
		case CS is
			when idle   =>
				ld_adr	 <= '1';
				if(Run_MIPL ='1') then 		NS <= wait_1  ;
				else                      	NS <= idle;
				end if; 
				
			when wait_1 => -- wait for next sample time
				if   (smp_flag ='1') then 	NS <= nextadr;
				elsif(Run_MIPL ='0')then 	NS <= idle;
				else                      	NS <= wait_1;
				end if;
				-- outputs --
				nclr_cnt <= '1'; -- not reset, but waiting
				nclr_adr <= '1';
				inc_adr  <= '0';
				
			when nextadr => 
				if   (end_flag ='1') then 	NS <= hold;
				elsif(Run_MIPL ='0') then 	NS <= idle;
				else                      	NS <= wait_1;
				end if;
				-- outputs --
				nclr_cnt <= '0'; -- clear the interval count
				nclr_adr <= '1';
				inc_adr  <= '1';  -- enable the addr count
				
			when hold => -- Hold last address until end of cycle trigger
				if(Run_MIPL ='0')   then 	NS <= idle;
				else                      	NS <= hold;
				end if;
				-- outputs --
				nclr_cnt <= '1';
				nclr_adr <= '1';
				inc_adr <= '0';
				
		end case; 
				
	end process; -- sm_combinatorial

-- Update Strobe Out ---------------------------------------------------
	STROBE_DELAY: 
	process (clk, nreset)
	begin
		if   (nreset = '0') then strb_delay <= (others => '0');
		elsif rising_edge(clk) then
			strb_delay(0) <= Phase_Err_Strb;
			strb_delay(1) <= strb_delay(0);
			Update_Strobe <= strb_delay(1);
		end if;
	end process;

end main;

