--------------------------------------------------------------------
--  This routine is used to manage the function of a 
--  momentary pushbutton.  I will initiate a "save all"
--  to ROM memory task when push briefly and it will
--  generate a syncronized board reset if pushed for
--  a longer duration.

Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE work.pkg_081006.all;  -- my definitions and entities

-------------------------------------------------------------------- 

entity Reset_Controller is
	port(clk   		: in std_logic;
		 Reset_In	: in std_logic;
		 PB_Reset	: in std_logic;
		 
		 nreset_out : out std_logic
		);
end Reset_Controller;
	
architecture main of Reset_Controller is
	signal syncd_reset	: std_logic;
	signal pressed 		: std_logic;
	
	-- Counter/Timer declarations --------------------------------------
	signal timer	 	: std_logic_vector(31 downto 0);
	signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, go);
	signal CS, NS  : STATE_TYPE;
	

begin -- main --------------------------------------------------------
		
	RESET_SYNC: led  -- Syncronizes the register power on reset
	port map(sig_in => Reset_In, clk => clk, clrn => '1', pls_out => syncd_reset );
	
	PB_RESET_SYNC: led  -- Syncronizes the pushbutton reset
	port map(sig_in => PB_Reset, clk => clk, clrn => '1', pls_out => pressed );
	
	TIMER_PROCESS:
	process (clk, timer, timer_ena)
	begin
		if  rising_edge(clk) then
			if   (timer_ena = '0') then timer <= (others => '0');
			else						timer <= timer + 1;
			end if;
		end if;
	end process;
	
-- State Machine definition ---------------------------
	sm_reg: process (clk, syncd_reset)	
	begin
		if(syncd_reset = '1')       then CS <= init;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, timer, pressed)
	begin
		-- defaults ---------------
		timer_ena 	<= '0';
		nreset_out  <= '1';
		---------------------------
	
		case CS is
			when init   =>			
				nreset_out  <= '0'; -- active low
				timer_ena 	<= '1';
				if   (timer < X"00186A00") then	NS <= init; -- 20ms @ clk=80MHz
				else							NS <= go;
				end if;
				
			when go     =>
				if(pressed = '1')then 			NS <= init;
				else           	  				NS <= go;
				end if;
				
			when others =>						NS <= init;
		end case; 
				
	end process; -- sm_combinatorial

end main;

