Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity Start_Stop_Logic is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Start_Delay	: in std_logic_vector(15 downto 0); --12.5ns per count
		 Start_Trig		: in std_logic;
		 Beam			: in std_logic;
		 BeamGate_nInjSync : in std_logic;
		 
		 Beam_Gate		: out std_logic;
 		 HOLD_Acc_PD	: out std_logic;		 
 		 Beam_Cyc_Reset	: out std_logic

		);
end Start_Stop_Logic;
	
architecture main of Start_Stop_Logic is

signal start			: std_logic;
signal syncd_Beam   	: std_logic;
signal no_delay			: std_logic;

signal delay		: std_logic_vector(23 downto 0);

signal timer		: std_logic_vector(23 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, wait1, wait_over, cycle_start, cycle_active, hold_pd_out, no_beam, send_reset);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

delay <= X"0" & Start_Delay(15 downto 0) & X"0";

sync_Start_Trig: led
port map(sig_in => Start_Trig, clk => clk, clrn => nreset, pls_out => start);

process(clk, nreset, Beam, BeamGate_nInjSync)
begin
	if (nreset = '0') then syncd_Beam <= '0';
	elsif rising_edge(clk) then
		syncd_Beam 	<= Beam;
		no_delay	<= BeamGate_nInjSync;
	end if;
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then 	timer <= (others => '0');
		else						timer <= timer + 1;
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, start, timer, syncd_Beam, delay, no_delay)
begin
	-- default outputs --
	timer_ena 		<= '1'; 
	Beam_Gate		<= '0';
	HOLD_Acc_PD		<= '0';
	Beam_Cyc_Reset	<= '0';
		
	case CS is
		when idle   =>
			timer_ena		<= '0';   			
			if (start = '1') 	then		
				if   (no_delay = '1') then	NS <= cycle_start;
				else						NS <= wait1;
				end if;
			else							NS <= idle;
			end if;
	
		when wait1   =>
									-- wait for delay from trigger to end
			if(timer = delay)          then	NS <= wait_over;
			elsif(timer = X"30D400")   then	NS <= send_reset; -- just in case delay is set wrong
			else							NS <= wait1;
			end if;
		
		when wait_over =>					NS <= cycle_start;
			timer_ena		<= '0';  -- reset the timer 			
		
		when cycle_start  =>
			Beam_Gate		<= '1';
									-- we hope to see beam 500us after Injection Sync
			if   (syncd_Beam = '1')  then	NS <= cycle_active; 
			elsif(timer = X"04E200") then	NS <= send_reset; -- 4 ms --previous-- time out at 0.5 ms
			else							NS <= cycle_start;
			end if;

		when cycle_active  =>
			Beam_Gate		<= '1';
									 -- we expect beam for full cycle 
			if   (syncd_Beam = '0') then	NS <= hold_pd_out;
			elsif(timer = X"30D400")   then	NS <= send_reset; -- time out at 40 ms
			else							NS <= cycle_active;
			end if;

		when hold_pd_out  =>				
			Beam_Gate		<= '1';
			HOLD_Acc_PD		<= '1'; -- hold the acceleration PD before indicating no beam
									-- check timer just in case of major coincidence
			if(timer = X"30D400")   then	NS <= send_reset; -- time out at 40 ms
			else							NS <= no_beam;
			end if;
			
		when no_beam  =>
			Beam_Gate		<= '0';
			HOLD_Acc_PD		<= '1'; -- hold the acceleration PD until reset
									-- we reset ready for next cycle after 40 ms
			if(timer = X"30D400")   then	NS <= send_reset; -- time out at 40 ms
			else							NS <= no_beam;
			end if;
			
		when send_reset  =>					NS <= idle;
			Beam_Cyc_Reset	<= '1';
			
		when others =>						NS <= idle;
	end case; 
			
end process;

end main;