Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity UP_DN_Filter is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Input			: in std_logic;
		 
		 Output			: out std_logic		 
		);
end UP_DN_Filter;
	
architecture main of UP_DN_Filter is

signal up			: std_logic;
signal dn			: std_logic;
signal edge			: std_logic;

signal filter		: std_logic;
signal ena_input	: std_logic;

signal timer		: std_logic_vector(15 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(hold, switch);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
sync_UP: led
port map(sig_in => Input, clk => clk, clrn => nreset, pls_out => up);
	
sync_DN: led
port map(sig_in => not Input, clk => clk, clrn => nreset, pls_out => dn);

	edge <= up OR dn;

FILTER_REGISTER:
process(clk, nreset, Input, ena_input)
begin
	if   (nreset = '0')    then filter <= '0';
	elsif rising_edge(clk) then 
		if   (ena_input = '1') then filter <= Input;
		else						filter <= filter;
		end if;
	end if;
end process;

	Output	<= filter; 

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= hold;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, edge)
begin
		timer_ena	<= '0';
		
		case CS is
		when hold   =>
			ena_input 	<= '0';
			timer_ena	<= '1';
			if   (timer = X"0C80") then	NS <= switch;
			else						NS <= hold;
			end if;
	
		when switch  =>
			ena_input 	<= '1';
			if (edge = '1') then 	NS <= hold;
			else					NS <= switch;
			end if;
	
		when others =>				NS <= hold;
	end case; 
			
end process;

end main;