Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity UP_DN_Filter_v2 is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Input			: in std_logic;
		 delay			: in std_logic_vector(15 downto 0);
		 
		 Output			: out std_logic		 
		);
end UP_DN_Filter_v2;
	
architecture main of UP_DN_Filter_v2 is

signal up			: std_logic;
signal dn			: std_logic;

signal timer		: std_logic_vector(15 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, up_state, dn_state, up_check, dn_check);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
sync_UP: led
port map(sig_in => Input, clk => clk, clrn => nreset, pls_out => up);
	
sync_DN: led
port map(sig_in => not Input, clk => clk, clrn => nreset, pls_out => dn);


PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= init;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, up, dn, delay)
begin
		timer_ena	<= '0';
		
		case CS is
		when init   =>
			Output <= '1';
			if(up = '1') then 			NS <= up_state;
			else		 	 			NS <= dn_state;
			end if;
		
		when up_state =>
			Output <= '1';
			
			if(dn = '1') then 			NS <= dn_check;
			else						NS <= up_state;
			end if;

		when dn_check =>
			Output <= '1';

			timer_ena	<= '1';
			if   (timer = delay) then	NS <= dn_state;
			elsif(up = '1') then		NS <= up_state;
			else						NS <= dn_check;
			end if;
	
		when dn_state =>
			Output <= '0';
			
			if(up = '1') then 			NS <= up_check;
			else						NS <= dn_state;
			end if;

		when up_check =>
			Output <= '0';

			timer_ena	<= '1';
			if   (timer = delay) then	NS <= up_state;
			elsif(dn = '1') then		NS <= dn_state;
			else						NS <= up_check;
			end if;
	
		when others =>					NS <= init;
	end case; 
			
end process;

end main;