Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity ZC_Timer is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 start_measurement : in std_logic;
		 stop_measurement  : in std_logic;
		 polarity		: in std_logic;
		 
		 ZC_Time		: out std_logic_vector(15 downto 0)
		);
end ZC_Timer;
	
architecture main of ZC_Timer is

signal up			: std_logic;
signal dn			: std_logic;
signal strt			: std_logic;
signal stop			: std_logic;
signal latch_time	: std_logic;

signal timer		: std_logic_vector(19 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, strt_timer, run_timer, latch_timer);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
sync_UP: led
port map(sig_in => polarity, clk => clk, clrn => nreset, pls_out => up);
	
sync_DN: led
port map(sig_in => not polarity, clk => clk, clrn => nreset, pls_out => dn);

sync_START: led
port map(sig_in => start_measurement, clk => clk, clrn => nreset, pls_out => strt);
	
sync_STOP: led
port map(sig_in => stop_measurement, clk => clk, clrn => nreset, pls_out => stop);

---------------------------------------------------------------------
PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then 	timer <= timer + 1;
		else						timer <= (others => '0');
		end if;
	end if;
end process;

---------------------------------------------------------------------
TIMER_LATCH: process(clk, nreset, timer, latch_time)
begin
	if  (nreset = '0') then 		ZC_Time <= (others => '0');
	elsif rising_edge(clk) then
		if  (latch_time = '1') then 	ZC_Time <= timer(18 downto 3);
		end if;
	end if;	
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if (nreset='0')             then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, up, dn, strt, stop)
begin
	timer_ena	<= '0';
	latch_time	<= '0';	
	case CS is
	when idle =>
		if(strt = '1') then			NS <= strt_timer;
		else						NS <= idle;
		end if;
		
	when strt_timer =>
		if(up = '1') then			NS <= run_timer;
		elsif(stop = '1') then		NS <= idle;
		else						NS <= strt_timer;
		end if;
			
	when run_timer =>
		timer_ena	<= '1';
		if   (dn = '1')  then 		NS <= latch_timer;
		elsif(stop = '1') then		NS <= idle;
		else						NS <= run_timer;
		end if;

	when latch_timer =>				NS <= strt_timer;
		timer_ena	<= '1'; 
		latch_time	<= '1';
		
		if(stop = '1')	 then		NS <= idle;
		else						NS <= strt_timer;
		end if;
	
	when others =>					NS <= idle;
	end case;			
end process;

end main;